local _, BigBrother = ...

local module = BigBrother:NewModule("Checks", "AceTimer-3.0")
if not module then return end

local L = LibStub("AceLocale-3.0"):GetLocale("BigBrother")
local SpellData = BigBrother.SpellData

do -- BigBrother:SendChatMessageList(title, list, channel)
	local chatTypes = {
		SELF = true,
		SAY = true,
		GROUP = true,
		PARTY = true,
		GUILD = true,
		OFFICIER = true,
		RAID = true,
		RAID_WARNING = true,
		WHISPER = true,
		CHANNEL = true,
	}

	--pick the right output or fallback to self
	local function getChannel(channel)
		channel = channel and channel:upper() or BigBrother:GetModule("Checks").db.profile.outputDefault:upper()

		--seems like a dirty way to do this ;[ makes the syntax /bb qc channel:mychannel or /run BigBrother:ConsumableCheck("channel:mychannel")
		local channelName
		channel, channelName = strsplit(":", channel, 2)

		if not chatTypes[channel] then
			return "SELF"
		end

		if channel == "CHANNEL" then
			local index = GetChannelName(channelName)
			if index > 0 then
				return channel, index
			else
				return "SELF"
			end
		elseif channel == "WHISPER" then
			if UnitCanCooperate("player", "target") then
				return channel, UnitName("target")
			else
				return "SELF"
			end
		elseif channel == "GROUP" then
			if IsInRaid() then
				channel = "RAID"
			elseif IsInGroup() then
				channel = "PARTY"
			else
				return "SELF"
			end
		end
		if channel == "RAID" or channel == "PARTY" then
			if IsPartyLFG() then
				return "INSTANCE_CHAT"
			elseif not IsInGroup() then
				return "SELF"
			end
		end

		return channel
	end

	function module:SendChatMessage(text, channel)
		if type(text) ~= "string" or text == "" then return end

		local target
		channel, target = getChannel(channel)
		if not channel then return end

		if channel == "SELF" then
			print(format("|Hbigbrother:%s|h|cff33ff99BigBrother|r|h: %s", text, text))
		else
			SendChatMessage(text, channel, nil, target)
		end
	end

	function module:SendChatMessageList(title, list, channel)
		if type(title) ~= "string" or title == "" then return end
		if type(list) ~= "table" or #list == 0 then return end

		local target
		channel, target = getChannel(channel)
		if not channel then return end

		table.sort(list)
		local names = table.concat(list, ", ")
		local text = ("%s (%d): %s"):format(title, #list, names)

		if channel == "SELF" then
			local coloredText = ("|cffff8040%s (%d)|r: %s"):format(title, #list, names)
			print(format("|Hbigbrother:%s|h|cff33ff99BigBrother|r|h: %s", text, coloredText))
		else
			SendChatMessage(text, channel, nil, target)
		end
	end
end

do
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", function(self, event, msg)
		if msg:sub(1, 11) == "BigBrother>" then return true end
	end)

	local function whisper(name, text)
		if not module.db.profile.blame then return end
		ChatThrottleLib:SendChatMessage("BULK", "", ("BigBrother> %s"):format(text), "WHISPER", nil, name)
	end

	function module:ConsumableCheck(channel)
		local groups = BigBrother:GetCheckedGroups()
		local inRaid = IsInRaid()

		local missingFlaskList = {}
		local missingElixirList = {}
		local missingFoodList = {}

		for unit, index in BigBrother:IterateGroup() do
			local name, realm = UnitName(unit)
			if not name then break end
			local shortName = name
			if realm then name = name.."-"..realm end

			local subgroup = inRaid and select(3, GetRaidRosterInfo(index)) or 1
			local online = UnitIsConnected(unit)
			local dead = UnitIsDeadOrGhost(unit)

			if online and not dead and groups[subgroup] then

				--check flasks/elixirs
				if self.db.profile.checkFlasks then
					local numElixirs = 0
					for _, spellName in next, SpellData.elixirsBattle do
						if UnitBuff(unit, spellName) then
							numElixirs = numElixirs + 1
							break
						end
					end
					for _, spellName in next, SpellData.elixirsGuardian do
						if UnitBuff(unit, spellName) then
							numElixirs = numElixirs + 1
							break
						end
					end

					if numElixirs == 1 then
						whisper(name, L["One Elixir"])
						missingElixirList[#missingElixirList + 1] = shortName
					elseif numElixirs == 0 then
						local found = nil
						for _, spellName in next, SpellData.flasks do
							if UnitBuff(unit, spellName) then
								found = true
								break
							end
						end
						if not found then
							whisper(name, L["No Flask"])
							missingFlaskList[#missingFlaskList + 1] = shortName
						end
					end
				end

				--check food
				if self.db.profile.checkFood then
					local found = nil
					for _, spellName in next, SpellData.foods do
						local id = select(11, UnitBuff(unit, spellName))
						if id and (not self.db.profile.maxOnly or SpellData.foodsMax[id]) then
							found = true
							break
						end
					end
					if not found then
						whisper(name, L["Not Well Fed"])
						missingFoodList[#missingFoodList + 1] = shortName
					end
				end

			end --if
		end --for

		if #missingElixirList > 0 then
			self:SendChatMessageList(L["One Elixir"], missingElixirList, channel)
		end
		if #missingFlaskList > 0 then
			self:SendChatMessageList(L["No Flask"], missingFlaskList, channel)
		end
		if #missingFoodList > 0 then
			self:SendChatMessageList(L["Not Well Fed"], missingFoodList, channel)
		end
	end
end


do --BigBrother:RaidBuffCheck(channel)
	local missing, list, aura = {}, {}, {}
	function module:RaidBuffCheck(channel)
		if not self.db.profile.checkBuffs then return end
		local playerList = BigBrother.UpdateRosterBuffs(true)
		local availableRaidBuffs = BigBrother.UpdateAvailableRaidBuffs()
		wipe(missing)
		wipe(list)
		wipe(aura)

		for _,player in next, playerList do
			for i=1,NUM_LE_RAID_BUFF_TYPES do
				local spellId = player.buff[i+3]
				if not spellId and availableRaidBuffs[i] then
					missing[i] = true
				elseif spellId and select(9, GetSpellInfo(spellId)) == 0 then --9th return is max cast range
					--don't complain about people that will have the buff when they aren't too far away
					aura[i] = true
				end
			end
		end

		for k,v in next, missing do
			if not aura[k] then
				list[#list+1] = _G["RAID_BUFF_"..k]
			end
		end
		if #list > 0 then
			self:SendChatMessageList(L["Missing Buffs"], list, channel)
		end
	end
end

function BigBrother:BuffCheck(channel)
	module:ConsumableCheck(channel)
	module:RaidBuffCheck(channel)
end

function module:READY_CHECK(_, sender)
	-- unchecked (0), checked (1), greyed (2)
	if not IsInGroup() or IsPartyLFG() or self.db.profile.checkReadyCheck == 0 then return end

	if self.db.profile.checkReadyCheck == 2 or (self.db.profile.checkReadyCheck == 1 and UnitIsUnit(sender, "player")) then
		BigBrother:BuffCheck()
	end
end


---------------------------------------
-- Init

function module:OnInitialize()
	self.db = BigBrother.db:RegisterNamespace("Checks", {
		profile = {
			checkFlasks = true,
			checkFood = true,
			checkBuffs = true,
			outputDefault = "raid",
			blame = false,
			checkReadyCheck = 1, --started by you
			maxOnly = false,
		}
	})
end

function module:OnEnable()
	self:RegisterEvent("READY_CHECK")
end


---------------------------------------
-- Options


function module:AddOptions(options)
	options.args.check_button = {
		name = L["Buff Check"],
		desc = L["Report who is missing consumables and/or buffs."],
		type = "execute",
		func = function() BigBrother:BuffCheck() end,
		order = 5,
	}

	local db = self.db.profile
	options.args.checks = {
		name = L["Checks"],
		--desc = L["Checks"],
		type = "group",
		get = function(info) return db[info[#info]] end,
		set = function(info, value) db[info[#info]] = value end,
		order = 20,
		order = 1,
		args = {
			checks = {
				name = L["Checks"],
				desc = L["Set what is included in a buff check."],
				type = "group",
				inline = true,
				order = 10,
				args = {
					checkFlasks = {
						name = L["Flasks/Elixirs"],
						desc = L["Include flasks and elixirs in checks."],
						type = "toggle",
						order = 5,
					},
					checkFood = {
						name = L["Food Buffs"],
						desc = L["Include food buffs in checks."],
						type = "toggle",
						order = 15,
					},
					checkBuffs = {
						name = L["Raid Buffs"],
						desc = L["Include raid buffs in checks."],
						type = "toggle",
						order = 20,
					},
				},
			}, --checks

			output = {
				name = L["Default Output"],
				desc = L["Set the default check output."],
				type = "group",
				inline = true,
				get = function(info)
					local key = info[#info]
					return string.gsub(db.outputDefault, "(.-):(.*)", "%1") == key
				end,
				set = function(info, value)
					if value then
						local key = info[#info]
						db.outputDefault = key
					end
				end,
				order = 20,
				args = {
					self = {
						name = L["Self"],
						desc = L["Print to your chat window."],
						type = "toggle",
						order = 5,
					},
					say = {
						name = L["Say"],
						desc = L["Report to say."],
						type = "toggle",
						order = 8,
					},
					group = {
						name = L["Group"],
						desc = L["Report to party or raid chat."],
						type = "toggle",
						order = 10,
					},
					party = {
						name = L["Party"],
						desc = L["Report to party chat."],
						type = "toggle",
						order = 11,
					},
					raid = {
						name = L["Raid"],
						desc = L["Report to raid chat."],
						type = "toggle",
						order = 12,
					},
					raid_warning = {
						name = L["Raid Warning"],
						desc = L["Report as a raid warning."],
						type = "toggle",
						order = 13,
					},
					guild = {
						name = L["Guild"],
						desc = L["Report to guild chat."],
						type = "toggle",
						order = 15,
					},
					officer = {
						name = L["Officer"],
						desc = L["Report to officer chat."],
						type = "toggle",
						order = 20,
					},
					channel = {
						name = L["Channel"],
						desc = L["Report to a custom channel."],
						type = "toggle",
						order = 25,
					},
					extra = { --pretty silly implementation, but oh well
						name = L["Channel"],
						type = "select",
						values = function() BigBrother:GetChatChannels() end,
						get = function()
							local channel = string.gsub(db.outputDefault, "(.-):(.*)", "%2")
							return GetChannelName(channel) > 0 and channel or ""
						end,
						set = function(info, value)
							db.outputDefault = "channel:"..value
						end,
						hidden = function() return string.gsub(db.outputDefault, "(.-):(.*)", "%1") ~= "channel" end,
						order = 101,
					},
				},
			},

			blame = {
				name = L["Whisper Missing"],
				desc = L["Whisper players that are missing consumables on checks."],
				type = "toggle",
				order = 25,
			},
			checkReadyCheck = {
				name = L["Check on Ready Check"],
				desc = L["Buff check on a ready check when in a non-LFG instance group.\n\n|cffffff33Checked|r: Report if started by you.\n|cffccccccGray|r: Report always."],
				type = "toggle",
				tristate = true,
				-- unchecked (false/0), checked (true/1), greyed (nil/2)
				get = function()
					local value = db.checkReadyCheck
					if value == 0 then
						return false
					elseif value == 1 then
						return true
					elseif value == 2 then
						return nil
					end
				end,
				set = function(info, value)
					if value == false then
						value = 0
					elseif value == true then
						value = 1
					elseif value == nil then
						value = 2
					end
					db.checkReadyCheck = value
				end,
				order = 30,
			},
			maxOnly = {
				name = L["Max Quality Only"],
				desc = L["Only count consumables of the highest available stat value in checks."],
				type = "toggle",
				order = 35,
			},
		},
	}
end

